/*****************************************
*                                        *
*        TP CSI2 - ESSI3 1998/99         *
*                                        *
*    CAUVIN Florian - NAVARRO Mikael     *
*                                        *
*          PROGRAMME PRINCIPAL           *
*                                        *
*****************************************/

#include <stdio.h>
#include <math.h>
#include "maillage.h"
#include "solvellip.h"

int main() {
  Maillage Avion;
  SolvEllip Solveur;

  int ni,NbIter=25;  /* Nb iterations ("non lineaires") du maillage */
  int ngs,NbGS=200;  /* Nb d'iterations ("lineaires") de GS */
  int j,k;
  int n,Nb=60;          /* Nb d'iterations du calcul de l'ecoulement */
  int nrgs,NbResGS=2000;  /* Nb di'iterations de GS pr ecoulement */
  double res,reslin,res1,res2;
  double res_init,res1_init,res2_init=0.0;
  char* FichierInitial="maillage.gnu";
  char* FichierFinal="ecoulement.gnu";
  char* FichierFinalMaple="ecoulementmpl";
  FILE *fichier, *fichier2;

  double** Phi;

  /* Partie 1 : MAILLAGE */
  printf("Lecture du maillage initial : ");
  CreateMaillage(&Avion, FichierInitial);
  printf(" ok !\n\n");
  CreateSolvEllip(&Solveur,&Avion);

  printf("Calcul du maillage \n");
  Visualize(&Avion,"","");

  fichier  = fopen("residu_non_lineaire.gnu", "w");
  fichier2 = fopen("residu_lineaire.gnu","w");

  /* residu initiaux */
  CalculCoefA(&Solveur);
  res1_init=Residu(&Solveur, (Avion).x);
  res2_init=Residu(&Solveur, (Avion).y);
  res_init=sqrt(res1_init+res2_init);

  /* Iterations non lineaires */
  for (ni=1;ni<=NbIter;ni++) {
    CalculCoefA(&Solveur);
    res1=Residu(&Solveur, (Avion).x);
    res2=Residu(&Solveur, (Avion).y);
    res=sqrt(res1+res2);

    /*residu non lineaire relatif*/
    fprintf(fichier, "%d %lf\n", ni, log10(res/res_init));

    /* Iterations lineaires */
    for(ngs=1; ngs<=NbGS; ngs++) {
      IterGaussSeidel(&Solveur, (Avion).x);
      IterGaussSeidel(&Solveur, (Avion).y);
      
      /* residu lineaire a la dixieme iteration */
      if(ni==10) {
	res1=Residu(&Solveur, (Avion).x);
	res2=Residu(&Solveur, (Avion).y);
	reslin=sqrt(res1+res2);
	fprintf(fichier2, "%d %lf\n", ngs, log10(reslin/res_init));
      }

    }
//    Visualize(&Avion,"","");
    
    printf("iteration %i : ",ni);
    printf("Residu = %lf",res);
    if (ni==10)
      printf("  -> calcul du residu \"lineaire\"");
    printf("\n");

  }
  fclose(fichier);
  fclose(fichier2);

  /* Partie 2 : CALCUL DE L'ECOULEMENT */

  fichier=fopen("residu_ecoulement.gnu","w");
  printf("\nCalcul de l'ecoulement\n");  

  /* Calcul de alpha, beta et gamma a l'interieur */
  CalculCoefA(&Solveur);
  /* Calcul de beta et gamma sur la frontiere */
  /* Frontiere d */
  CalculCoefD(&Solveur);
  /* Frontiere c */
  CalculCoefC(&Solveur);
  /* Frontiere b */
  CalculCoefB(&Solveur);

  /* Initialisation de Phi a 0 */
  Phi = (double**) malloc(sizeof(double*) * ((Avion).jm+1));
  for (j=0; j<=(Avion).jm; j++)
    Phi[j] =(double*) malloc(sizeof(double) * ((Avion).km+1));

  for(j=0; j<=(Avion).jm; j++)
    for(k=0; k<=(Avion).km; k++)
      Phi[j][k] = 0;



  /* Boucle iterative */
  for(n=1; n<Nb; n++) {
    /* Reactualisation de la frontiere */
    /* Frontiere a */
    ResGlissementA(&Solveur, Phi);
    /* Frontiere d */
    ResNeumannD(&Solveur, Phi);
    /* Frontiere c */
    ResNeumannC(&Solveur, Phi);
    /* Frontiere b */
    ResNeumannB(&Solveur, Phi);






    
    /* Resolution du probleme a l'interieur */
    for(nrgs=1; nrgs<=NbResGS; nrgs++) {
      IterGaussSeidel(&Solveur, Phi);
    }

    if (n==1) {
      res_init=sqrt(Residu(&Solveur, Phi));
    }
    res=sqrt(Residu(&Solveur, Phi));
    /*residu non lineaire relatif*/
    fprintf(fichier, "%d %lf\n", n, log10(res/res_init));
  } /* fin boucle iterative */    



  //Visualize(&Avion, "", "");

  fclose(fichier);
  /* Sauvegarde de la solution pour gnuplot */
  /* sous gnuplot taper splot 'ecoulement.gnu' */
  fichier=fopen(FichierFinal,"w");
  fprintf(fichier,"# %d %d\n",(Avion).jm,(Avion).km);
  for(k=0;k<=(Avion).km;k++) {
    fprintf(fichier,"\n");
    for(j=0;j<=(Avion).jm;j++)
      fprintf(fichier,"%lf %lf %lf\n",(Avion).x[j][k],(Avion).y[j][k], Phi[j][k]);
  }
  /* partie juste utile pour gnuplot */
  for(j=0;j<=(Avion).jm;j++) {
    fprintf(fichier,"\n");
    for(k=0;k<=(Avion).km;k++)
      fprintf(fichier,"%lf %lf %lf\n",(Avion).x[j][k],(Avion).y[j][k], Phi[j][k]);
  }
  fclose(fichier);

  /* Sauvegarde de la solution pour 'erable' */
  /* sous xmaple taper  with(plots):  et puis  read(ecoulementmpl); */
  fichier=fopen(FichierFinalMaple,"w");
  fprintf(fichier,"polygonplot3d([");
  for(k=0;k<=(Avion).km-1;k++) {
    fprintf(fichier,"\n");
    for(j=0;j<=(Avion).jm-1;j++) {
      fprintf(fichier,"[[%lf , %f , %lf],\n",(Avion).x[j][k],(Avion).y[j][k], Phi[j][k]);
      fprintf(fichier,"[%lf , %lf , %lf],\n",(Avion).x[j+1][k],(Avion).y[j+1][k], Phi[j+1][k
]);
      fprintf(fichier,"[%lf , %lf , %lf],\n",(Avion).x[j+1][k+1],(Avion).y[j+1][k+1], Phi[j+
1][k+1]);
      fprintf(fichier,"[%lf , %lf , %lf]],\n",(Avion).x[j][k+1],(Avion).y[j][k+1], Phi[j][k+
1]);
    }
  }
  /* ! enlever la derniere virgule */
  fprintf(fichier, "]);");
  fclose(fichier);

	
  printf("Calculs termines\n");
  /* Liberation de la memoire */
  DestroySolvEllip(&Solveur);
  DestroyMaillage(&Avion);

  return 0;
}
