/*****************************************
*                                        *
*        TP CSI2 - ESSI3 1998/99         *
*                                        *
*    CAUVIN Florian - NAVARRO Mikael     *
*                                        *
*          PROGRAMME PRINCIPAL           *
*                                        *
*****************************************/

#include <stdio.h>
#include <math.h>
#include "maillage.h"
#include "solvellip.h"


void allocPhi(Maillage* Avion,double **Phi) {
  /* Initialisation de Phi a 0 */
  int j;
  Phi = (double**) malloc(sizeof(double*) * ((Avion)->jm+1));
  for (j=0; j<=(Avion)->jm; j++)
    Phi[j] =(double*) malloc(sizeof(double) * ((Avion)->km+1));
}


void deraffiner(Maillage* fin,Maillage* grossier) {
  int j,k;

  grossier->jm=fin->jm/2;
  grossier->km=fin->km/2;
  grossier->nom=(char*)malloc(strlen(fin->nom));
  strcpy(grossier->nom,fin->nom);

  /* allocation de x et y */
  grossier->x=(double**) malloc(sizeof(double*) * (grossier->jm+1));
  grossier->y=(double**) malloc(sizeof(double*) * (grossier->jm+1));
  for (j=0;j<=grossier->jm;j++) {
    grossier->x[j]=(double*) malloc(sizeof(double) * (grossier->km+1));
    grossier->y[j]=(double*) malloc(sizeof(double) * (grossier->km+1));
  }

  /* remplissage de x et y grace au fichier */
  for(k=0;k<=grossier->km;k++) {
    for(j=0;j<=grossier->jm;j++) {
        grossier->x[j][k]=fin->x[j*2][k*2];
        grossier->y[j][k]=fin->y[j*2][k*2];
    }
  }
}



void prolonge(Maillage* grossier,double** sg,Maillage* fin,double** sf) {
  
  int j,k;
  //commun
  for (j=1;j<grossier->jm;j++)
   for(k=1;j<grossier->km;k++)
      sf[2*j][2*k]=sg[j][k];

  //les bords
  for (j=0;j<grossier->jm;j++) 
  {
    sf[2*j+1][0]=(sg[j][0]+sg[j+2][0])/2;
    sf[2*j+1][fin->km]=(sg[j][grossier->km]+sf[j+2][grossier->km])/2;
  }
  for (k=0;k<grossier->km;k++) 
  {
    sf[0][2*k+1]=(sg[0][k]+sg[0][k+2])/2;
    sf[fin->jm][2*k+1]=(sg[grossier->jm][k]+sf[grossier->jm][k+2])/2;
  }
  
  //
}


/* solveur elliptique pour le maillage */
void maille(SolvEllip *Solveur,Maillage* Avion) {

  int NbIter=25;
  int NbGS=50;
  int ni,ngs;

  /* Iterations non lineaires */
  for (ni=1;ni<=NbIter;ni++) {
    CalculCoefA(Solveur);

    /* Iterations lineaires */
    for(ngs=1; ngs<=NbGS; ngs++) {
      IterGaussSeidel(Solveur, (Avion)->x);
      IterGaussSeidel(Solveur, (Avion)->y);      
    }

  }
}


/* calcul ecoulement */
void resoud(SolvEllip *Solveur,Maillage* Avion,char* FichierMaple,double** Phi) {


  int Nb=5;
  int NbResGS=2000;
  int n,j,k,nrgs;
  FILE* fichier;

  /* calcul coef */
  CalculCoefA(Solveur);   /* interieur */
  CalculCoefD(Solveur);   /* Frontiere d */
  CalculCoefC(Solveur);   /* Frontiere c */
  CalculCoefB(Solveur);   /* Frontiere b */

//  res1=Residu(Solveur, (Avion)->x);
//  res2=Residu(Solveur, (Avion)->y);
//  res=sqrt(res1+res2);


  /* Boucle iterative */
  for(n=1; n<Nb; n++) {
    /* Reactualisation de la frontiere */
    ResGlissementA(Solveur, Phi); /* Frontiere a */
    ResNeumannD(Solveur, Phi);    /* Frontiere d */
    ResNeumannC(Solveur, Phi);    /* Frontiere c */
    ResNeumannB(Solveur, Phi);    /* Frontiere b */
    
    /* Resolution du probleme a l'interieur */
    for(nrgs=1; nrgs<=NbResGS; nrgs++) {
      IterGaussSeidel(Solveur, Phi);
    }

   }
   /* fin boucle iterative */
  //Visualize(&Avion, "", "");

  /* Sauvegarde de la solution pour gnuplot */
  /* sous gnuplot taper splot 'ecoulement.gnu'
  fichier=fopen(FichierFinal,"w");
  fprintf(fichier,"# %d %d\n",(Avion)->jm,(Avion)->km);
  for(k=0;k<=(Avion)->km;k++) {
    fprintf(fichier,"\n");
    for(j=0;j<=(Avion)->jm;j++)
      fprintf(fichier,"%lf %lf %lf\n",(Avion)->x[j][k],(Avion)->y[j][k], Phi[j][k]);
  }

  /* partie juste utile pour gnuplot
  for(j=0;j<=(Avion)->jm;j++) {
    fprintf(fichier,"\n");
    for(k=0;k<=(Avion)->km;k++)
      fprintf(fichier,"%lf %lf %lf\n",(Avion)->x[j][k],(Avion)->y[j][k], Phi[j][k]);
  }
  fclose(fichier);*/

  /* Sauvegarde de la solution pour 'erable' */
  /* sous xmaple taper  with(plots):  et puis  read(ecoulementmpl); */
  fichier=fopen(FichierMaple,"w");
  fprintf(fichier,"polygonplot3d([");
  for(k=0;k<=(Avion)->km-1;k++) {
    fprintf(fichier,"\n");
    for(j=0;j<=(Avion)->jm-1;j++) {
      fprintf(fichier,"[[%lf , %f , %lf],\n",(Avion)->x[j][k],(Avion)->y[j][k], Phi[j][k]);
      fprintf(fichier,"[%lf , %lf , %lf],\n",(Avion)->x[j+1][k],(Avion)->y[j+1][k], Phi[j+1][k
]);
      fprintf(fichier,"[%lf , %lf , %lf],\n",(Avion)->x[j+1][k+1],(Avion)->y[j+1][k+1], Phi[j+
1][k+1]);
      fprintf(fichier,"[%lf , %lf , %lf]],\n",(Avion)->x[j][k+1],(Avion)->y[j][k+1], Phi[j][k+
1]);
    }
  }
  /* ! enlever la derniere virgule */
  fprintf(fichier, "]);");
  fclose(fichier);

}


int main() {
  Maillage Mh,M2h,M4h,M8h;
  SolvEllip Sh,S2h,S4h,S8h;
  char *Fh="mMhmpl",*F2h="M2hmpl",*F4h="M4hmpl",*F8h="M8hmpl";
  double **Phih,**Phi2h,**Phi4h,**Phi8h;
  char* FichierInitial="maillage.gnu";
  int j,k;

  /* creation des maillages */
  CreateMaillage(&Mh, FichierInitial);
  CreateSolvEllip(&Sh,&Mh);
  allocPhi(&M2h,Phih);
  for(j=0; j<=Mh.jm; j++)
    for(k=0; k<=Mh.km; k++)
      Phih[j][k] = 0;  
  maille(&Sh,&Mh);

  deraffiner(&Mh,&M2h);
  CreateSolvEllip(&S2h,&M2h);
  allocPhi(&M2h,Phi2h);
  maille(&S2h,&M2h);

  deraffiner(&M2h,&M4h);
  CreateSolvEllip(&S4h,&M4h);
  allocPhi(&M4h,Phi4h);
  maille(&S4h,&M4h);

  deraffiner(&M4h,&M8h);
  CreateSolvEllip(&S8h,&M8h);
  allocPhi(&M8h,Phi8h);
  maille(&S8h,&M8h);


  /* resolution / prolongement */
  resoud(&S8h,&M8h,F8h,Phi8h);
  prolonge(&M8h,Phi8h,&M4h,Phi4h);

  resoud(&S4h,&M4h,F4h,Phi4h);
  prolonge(&M4h,Phi4h,&M2h,Phi2h);

  resoud(&S2h,&M2h,F2h,Phi2h);
  prolonge(&M2h,Phi2h,&Mh,Phih);

  resoud(&Sh,&Mh,Fh,Phih);


  
}

//  int ni,NbIter=25;  /* Nb iterations ("non lineaires") du maillage */
//  int ngs,NbGS=5;  /* Nb d'iterations ("lineaires") de GS */
//  int j,k;
//  int n,Nb=5;          /* Nb d'iterations du calcul de l'ecoulement */
//  int nrgs,NbResGS=2000;  /* Nb di'iterations de GS pr ecoulement */
//  double res,reslin,res1,res2;
//  double res_init,res1_init,res2_init=0.0;
//  char* FichierInitial="maillage.gnu";
//  char* FichierFinal="ecoulement.gnu";
//  char* FichierFinalMaple="ecoulementmpl";
//  FILE *fichier, *fichier2;

//  double** Phi;

