/*****************************************
*                                        *
*        TP CSI2 - ESSI3 1998/99         *
*                                        *
*    CAUVIN Florian - NAVARRO Mikael     *
*                                        *
* Construction d'un maillage elliptique  *
*                                        *
*****************************************/

#include "solvellip.h"
#include <math.h>


void CreateSolvEllip(SolvEllip *this,Maillage *maillage) {
  int j;

  this->maillage=maillage;  
  this->alpha = (double**) malloc(sizeof(double*) * (maillage->jm+1));
  this->beta  = (double**) malloc(sizeof(double*) * (maillage->jm+1));
  this->gamma = (double**) malloc(sizeof(double*) * (maillage->jm+1));

  for (j=0;j<=maillage->jm;j++) {
    this->alpha[j] =(double*) malloc(sizeof(double) * (maillage->km+1));
    this->beta[j]  =(double*) malloc(sizeof(double) * (maillage->km+1));
    this->gamma[j] =(double*) malloc(sizeof(double) * (maillage->km+1));
  }
}

void DestroySolvEllip(SolvEllip *this) {
  int j;

  for(j=0;j<=this->maillage->jm;j++) {
    free(this->alpha[j]);
    free(this->beta[j]);
    free(this->gamma[j]);
  }
  free(this->alpha);
  free(this->beta);
  free(this->gamma);
}

void Iteration(SolvEllip *this) {
  int i;

  CalculCoefA(this);
  for(i=1; i<=100; i++) {
    IterGaussSeidel(this,this->maillage->x);
    IterGaussSeidel(this,this->maillage->y);
  }
}


void CalculCoefA(SolvEllip *this) {
  int j,k;
  double dxn,dyn,dxe,dye;
  double** x=this->maillage->x;
  double** y=this->maillage->y;

  for(j=1;j<=this->maillage->jm-1;j++)
    for(k=1;k<=this->maillage->km-1;k++) {
      dxn = x[j][k+1]-x[j][k-1];
      dyn = y[j][k+1]-y[j][k-1];
      dxe = x[j+1][k]-x[j-1][k];
      dye = y[j+1][k]-y[j-1][k];
      this->alpha[j][k] = dxn*dxn + dyn*dyn;
      this->beta [j][k] = dxn*dxe + dyn*dye;
      this->gamma[j][k] = dxe*dxe + dye*dye;
    }
}

void CalculCoefD(SolvEllip *this) {
  int j, k;
  double Dxb, Dxc, Dxd, Dyb, Dyc, Dyd;
  double **x=this->maillage->x;
  double **y=this->maillage->y;

  /* Frontiere d */
  j=this->maillage->jm;
  for(k=1; k<=this->maillage->km-1; k++) {
    Dxb = x[j-1][k]-x[j][k];
    Dxc = x[j-1][k+1]-x[j][k];
    Dxd = x[j][k+1]-x[j][k];
    Dyb = y[j-1][k]-y[j][k];
    Dyc = y[j-1][k+1]-y[j][k];
    Dyd = y[j][k+1]-y[j][k];
    this->beta[j][k] = 1/(1-(Dxb*Dxd+Dyb*Dyd)/(Dxc*Dxd+Dyc*Dyd));
    this->gamma[j][k] = (1*Dxd*(Dxb*Dyc-Dxc*Dyb))/(Dxc*Dxd+Dyc*Dyd-(Dxb*Dxd+Dyb*Dyd));
  }
}

void CalculCoefC(SolvEllip *this) {
  int j, k;
  double Dxb, Dxc, Dxd, Dyb, Dyc, Dyd;
  double **x=this->maillage->x;
  double **y=this->maillage->y;

  /* Frontiere c */
  k=this->maillage->km;
  for(j=this->maillage->jm; j>=1; j--) {
    Dxb = x[j][k-1]-x[j][k];
    Dxc = x[j-1][k-1]-x[j][k];
    Dxd = x[j-1][k]-x[j][k];
    Dyb = y[j][k-1]-y[j][k];
    Dyc = y[j-1][k-1]-y[j][k];
    Dyd = y[j-1][k]-y[j][k];
    this->beta[j][k] = 1/(1-(Dxb*Dxd+Dyb*Dyd)/(Dxc*Dxd+Dyc*Dyd));
    this->gamma[j][k] = (1*Dxd*(Dxb*Dyc-Dxc*Dyb))/(Dxc*Dxd+Dyc*Dyd-(Dxb*Dxd+Dyb*Dyd));
  }
}

void CalculCoefB(SolvEllip *this) {
  int j, k;
  double Dxb, Dxc, Dxd, Dyb, Dyc, Dyd;
  double **x=this->maillage->x;
  double **y=this->maillage->y;

  /* Frontiere b */
  j=0;
  for(k=this->maillage->km; k>=1; k--) {
    Dxb = x[j+1][k]-x[j][k];
    Dxc = x[j+1][k-1]-x[j][k];
    Dxd = x[j][k-1]-x[j][k];
    Dyb = y[j][k]-y[j][k];
    Dyc = y[j+1][k-1]-y[j][k];
    Dyd = y[j][k-1]-y[j][k];
    this->beta[j][k] = 1/(1-(Dxb*Dxd+Dyb*Dyd)/(Dxc*Dxd+Dyc*Dyd));
    this->gamma[j][k] = (1*Dxd*(Dxb*Dyc-Dxc*Dyb))/(Dxc*Dxd+Dyc*Dyd-(Dxb*Dxd+Dyb*Dyd));
  }
}


void ResGlissementA(SolvEllip *this, double **u) {
  int j, k;

  /* Frontiere a */
  k=0;
  for(j=0; j<=this->maillage->jm; j++) {
    u[j][k] = 0;
  }
}

void ResNeumannD(SolvEllip *this, double **u) {
  int j, k;

  /* Frontiere d */
  j=this->maillage->jm;
  for(k=1; k<=this->maillage->km-1; k++) {
    u[j][k] = this->beta[j][k]*u[j-1][k]
      +(1-this->beta[j][k])*u[j-1][k+1]
      -this->gamma[j][k];
  }
}

void ResNeumannC(SolvEllip *this, double **u) {
  int j, k;

  /* Frontiere c */
  k=this->maillage->km;
  for(j=this->maillage->jm; j>=1; j--) {
    u[j][k] = this->beta[j][k]*u[j][k-1]
      +(1-this->beta[j][k])*u[j-1][k-1]
      -this->gamma[j][k];
  }
}

void ResNeumannB(SolvEllip *this, double **u) {
  int j, k;

  /* Frontiere B */
  j=0;
  for(k=this->maillage->km; k>=1; k--) {
    u[j][k] = this->beta[j][k]*u[j+1][k]
      +(1-this->beta[j][k])*u[j+1][k-1]
      -this->gamma[j][k];
  }
}

void IterGaussSeidel(SolvEllip *this, double** u) {
  int j,k;

  for (j=1;j<=this->maillage->jm-1;j++) {
    for (k=1;k<=this->maillage->km-1;k++) {
      /* formule de Gauss Seidel pour la matrice tridiagonnale */
      u[j][k]=(1/(2*(this->alpha[j][k]+this->gamma[j][k]-this->beta[j][k])))
	*((this->alpha[j][k]-this->beta[j][k])*(u[j+1][k]+u[j-1][k])
	  +(this->gamma[j][k]-this->beta[j][k])*(u[j][k+1]+u[j][k-1])
	  +this->beta[j][k]*(u[j-1][k+1]+u[j+1][k-1]));
    }
  }
}


//residu au carre
double Residu(SolvEllip *this,double** u) {
  double residu=0.0,tmp,coef;
  int j,k;

  for (j=1;j<=this->maillage->jm-1;j++) {
    for (k=1;k<=this->maillage->km-1;k++) {
      coef=-2*(this->alpha[j][k]+this->gamma[j][k]-this->beta[j][k]);
      tmp = (
          coef*u[j][k]
	  +(this->alpha[j][k]-this->beta[j][k]) * (u[j+1][k]+u[j-1][k])
	  +(this->gamma[j][k]-this->beta[j][k]) * (u[j][k+1]+u[j][k-1])
	  + this->beta[j][k] * (u[j-1][k+1]+u[j+1][k-1])
          ) /coef;
      residu+=tmp*tmp;
    }
  }
 return residu;
}
